@@ -146,7 +146,7 @@ def get_index_review_data_single_day(index: str, review_year: int, review_month:
         raise TypeError("The 'review_year' and 'review_month' arguments must be integer types.")
     if not isinstance(verbose, bool):
         raise TypeError("The 'verbose' argument must be a boolean type.")
-
+    date = date.replace("/", "-")
     url = "https://www.samsonrockapis.com/forecast_flows&index={}" \
           "&review_year={}&review_month={}&review_name={}&date={}".format(index, review_year, review_month,
                                                                           review_type, date)
@@ -196,7 +196,8 @@ def get_index_review_data_daterange(index: str, review_year: int, review_month:
         raise TypeError("The 'review_year' and 'review_month' arguments must be integer types.")
     if not isinstance(verbose, bool):
         raise TypeError("The 'verbose' argument must be a boolean type.")
-
+    start_date = start_date.replace("/", "-")
+    end_date = end_date.replace("/", "-")
     url = "https://www.samsonrockapis.com/forecast_flows&index={}&review_year={}" \
           "&review_month={}&review_name={}&start_date={}&end_date={}".format(index, review_year, review_month,
                                                                              review_type, start_date, end_date)
