@@ -13,7 +13,7 @@ def post_tracking_assumptions_to_database(dataset: pd.DataFrame, access_token: s
     :param dataset: a Pandas DataFrame containing the data to upload within a given schema. (pd.DataFrame)
     :param access_token: the access token of the user, must be an active access token. (str)
     :param verbose: determines if status is printed. Default=True. (bool)
-    :return: N/A
+    :return: failed_uploads: the information which failed to upload. (pd.DataFrame)
     """
     if not isinstance(dataset, pd.DataFrame):
         raise TypeError("The 'dataset' argument must be a Pandas DataFrame.")
@@ -21,6 +21,7 @@ def post_tracking_assumptions_to_database(dataset: pd.DataFrame, access_token: s
         raise TypeError("The 'access_token' must be a string type.")
     if not isinstance(verbose, bool):
         raise TypeError("The 'verbose' argument must be a boolean argument.")
+    failed_uploads = []
     dict_list = dataset.to_dict('records')
     url = "https://www.samsonrockapis.com/tracking/upload"
     header = {'Authorization': 'Bearer ' + access_token}
@@ -30,11 +31,14 @@ def post_tracking_assumptions_to_database(dataset: pd.DataFrame, access_token: s
             if verbose:
                 print("Status Code: {}, data posted to tracking table.".format(response.status_code))
         elif response.status_code in [401, 403]:
+            failed_uploads.append(item)
             if verbose:
                 print("Status Code: {}, user not authenticated for endpoint.".format(response.status_code))
         else:
+            failed_uploads.append(item)
             if verbose:
                 print("Status Code: {}, {}.".format(response.status_code, response.text))
+    return pd.DataFrame.from_records(failed_uploads)
 
 
 def get_all_tracking_assumptions(access_token: str, verbose: bool = True):
@@ -61,7 +65,7 @@ def get_all_tracking_assumptions(access_token: str, verbose: bool = True):
         return pd.read_json(response.text)
     elif response.status_code == 404:
         if verbose:
-            print("Status Code: {}, cannot find tracking data.".format(response.status_code))
+            print("Status Code: {}, {}.".format(response.status_code, response.text))
         return pd.DataFrame()
     elif response.status_code in [401, 403]:
         if verbose:
@@ -89,27 +93,7 @@ def get_index_tracking_assumption_history(index: str, access_token: str, verbose
         raise TypeError("The 'index' and 'access_token' arguments must be string types.")
     if not isinstance(verbose, bool):
         raise TypeError("The 'verbose' argument must be a boolean type.")
-    index_values = ['AEX', 'AMX', 'AMZ', 'AMZI', 'AS51', 'ASX', 'BEL20', 'CAC', 'CPQ', 'DAX', 'DJASD', 'DJCASD',
-                    'DJDVY', 'DJEMDIV', 'DJEPCSD', 'DJINET', 'DJSHMB', 'DJSMDQT', 'DJUSDIVT', 'EPRA-EUROPE',
-                    'FTSEMIB', 'FUDP', 'GDM', 'GDXJ', 'GEIS-AWCHINA', 'GEIS-AWDEVEUR', 'GEIS-AWEMEUR',
-                    'GEIS-AWEXCHINA', 'GEIS-AWJAPAN', 'GEIS-AWLATAM', 'GEIS-AWMEA', 'GEIS-AWNAM',
-                    'GEIS-GSCCHINA', 'GEIS-GSCDEVEUR', 'GEIS-GSCEMEUR', 'GEIS-GSCEXCHINA', 'GEIS-GSCJAPAN',
-                    'GEIS-GSCLATAM', 'GEIS-GSCMEA', 'GEIS-GSCNAM', 'HSCEI', 'HUR', 'IBEX', 'KOSPI2', 'MCMOT',
-                    'MCX', 'MDAX', 'MEXBOL', 'MOAT', 'MSCI-DM-SC-ASIAEXJAP', 'MSCI-DM-SC-CAN', 'MSCI-DM-SC-EMEA',
-                    'MSCI-DM-SC-JAPAN', 'MSCI-DM-SC-USA', 'MSCI-DM-STD-ASIAEXJAP', 'MSCI-DM-STD-CAN',
-                    'MSCI-DM-STD-EMEA', 'MSCI-DM-STD-JAPAN', 'MSCI-DM-STD-USA', 'MSCI-EM-SC-AMERICAS',
-                    'MSCI-EM-SC-ASIA', 'MSCI-EM-SC-EMEA', 'MSCI-EM-STD-AMERICAS', 'MSCI-EM-STD-ASIA',
-                    'MSCI-EM-STD-EMEA', 'MSCI-MINVOL-EUROPE', 'MSCI-MINVOL-USA', 'MSCI-MINVOL-EM',
-                    'MSCI-SRI-ASIAEXJAP', 'MSCI-SRI-CANADA', 'MSCI-SRI-EMEA', 'MSCI-SRI-JAPAN', 'MSCI-SRI-USA',
-                    'MSCI-ESG-ASIAEXJAP', 'MSCI-ESG-CANADA', 'MSCI-ESG-EMEA', 'MSCI-ESG-JAPAN', 'MSCI-ESG-USA',
-                    'NBI', 'NDX', 'OMX', 'RIY', 'RTY', 'SD3E', 'SD3P', 'SDAX', 'SMI', 'SMIM', 'SPEUHDA',
-                    'SPGTAQD', 'SPGTCED', 'SPGTIND', 'SPI', 'SPTSX', 'SPTSX60', 'SPTXDV', 'SX5E', 'SX5P',
-                    'SXXP', 'TecDAX', 'TOP40', 'TW50', 'UKX', 'XIN9I', 'EPRA-ASIA', 'EPRA-NAM', 'IXAROBK',
-                    'IXBRHLTK', 'MID', 'SML', 'SPX', 'STXTDSV', 'TWDP', 'NIKKEI', 'TOPIX', 'CRSP', 'TA-125',
-                    'TA-35', 'HSI', 'KLCI', 'STI', 'MUNR', 'BLOSSOM', 'OMX30', 'FBLSSRTR', 'MDYF', 'FR10',
-                    'FREM', 'MCVAT', 'ENDP']
-    if index not in index_values:
-        raise ValueError("The 'index' value must be one of {}".format(index_values))
+
     url = "https://www.samsonrockapis.com/tracking&index={}".format(index)
     header = {'Authorization': 'Bearer ' + access_token}
     response = requests.get(url, headers=header)
@@ -119,8 +103,7 @@ def get_index_tracking_assumption_history(index: str, access_token: str, verbose
         return pd.read_json(response.text)
     elif response.status_code == 404:
         if verbose:
-            print("Status Code: {}, cannot find tracking data for {}.".format(response.status_code,
-                                                                                  index))
+            print("Status Code: {}, {}.".format(response.status_code, response.text))
         return pd.DataFrame()
     elif response.status_code in [401, 403]:
         if verbose:
@@ -148,27 +131,7 @@ def get_most_recent_index_tracking_assumption(index: str, access_token: str, ver
         raise TypeError("The 'index' and 'access_token' arguments must be string types.")
     if not isinstance(verbose, bool):
         raise TypeError("The 'verbose' argument must be a boolean type.")
-    index_values = ['AEX', 'AMX', 'AMZ', 'AMZI', 'AS51', 'ASX', 'BEL20', 'CAC', 'CPQ', 'DAX', 'DJASD', 'DJCASD',
-                    'DJDVY', 'DJEMDIV', 'DJEPCSD', 'DJINET', 'DJSHMB', 'DJSMDQT', 'DJUSDIVT', 'EPRA-EUROPE',
-                    'FTSEMIB', 'FUDP', 'GDM', 'GDXJ', 'GEIS-AWCHINA', 'GEIS-AWDEVEUR', 'GEIS-AWEMEUR',
-                    'GEIS-AWEXCHINA', 'GEIS-AWJAPAN', 'GEIS-AWLATAM', 'GEIS-AWMEA', 'GEIS-AWNAM',
-                    'GEIS-GSCCHINA', 'GEIS-GSCDEVEUR', 'GEIS-GSCEMEUR', 'GEIS-GSCEXCHINA', 'GEIS-GSCJAPAN',
-                    'GEIS-GSCLATAM', 'GEIS-GSCMEA', 'GEIS-GSCNAM', 'HSCEI', 'HUR', 'IBEX', 'KOSPI2', 'MCMOT',
-                    'MCX', 'MDAX', 'MEXBOL', 'MOAT', 'MSCI-DM-SC-ASIAEXJAP', 'MSCI-DM-SC-CAN', 'MSCI-DM-SC-EMEA',
-                    'MSCI-DM-SC-JAPAN', 'MSCI-DM-SC-USA', 'MSCI-DM-STD-ASIAEXJAP', 'MSCI-DM-STD-CAN',
-                    'MSCI-DM-STD-EMEA', 'MSCI-DM-STD-JAPAN', 'MSCI-DM-STD-USA', 'MSCI-EM-SC-AMERICAS',
-                    'MSCI-EM-SC-ASIA', 'MSCI-EM-SC-EMEA', 'MSCI-EM-STD-AMERICAS', 'MSCI-EM-STD-ASIA',
-                    'MSCI-EM-STD-EMEA', 'MSCI-MINVOL-EUROPE', 'MSCI-MINVOL-USA', 'MSCI-MINVOL-EM',
-                    'MSCI-SRI-ASIAEXJAP', 'MSCI-SRI-CANADA', 'MSCI-SRI-EMEA', 'MSCI-SRI-JAPAN', 'MSCI-SRI-USA',
-                    'MSCI-ESG-ASIAEXJAP', 'MSCI-ESG-CANADA', 'MSCI-ESG-EMEA', 'MSCI-ESG-JAPAN', 'MSCI-ESG-USA',
-                    'NBI', 'NDX', 'OMX', 'RIY', 'RTY', 'SD3E', 'SD3P', 'SDAX', 'SMI', 'SMIM', 'SPEUHDA',
-                    'SPGTAQD', 'SPGTCED', 'SPGTIND', 'SPI', 'SPTSX', 'SPTSX60', 'SPTXDV', 'SX5E', 'SX5P',
-                    'SXXP', 'TecDAX', 'TOP40', 'TW50', 'UKX', 'XIN9I', 'EPRA-ASIA', 'EPRA-NAM', 'IXAROBK',
-                    'IXBRHLTK', 'MID', 'SML', 'SPX', 'STXTDSV', 'TWDP', 'NIKKEI', 'TOPIX', 'CRSP', 'TA-125',
-                    'TA-35', 'HSI', 'KLCI', 'STI', 'MUNR', 'BLOSSOM', 'OMX30', 'FBLSSRTR', 'MDYF', 'FR10',
-                    'FREM', 'MCVAT', 'ENDP']
-    if index not in index_values:
-        raise ValueError("The 'index' value must be one of {}".format(index_values))
+
     url = "https://www.samsonrockapis.com/tracking&index={}&latest".format(index)
     header = {'Authorization': 'Bearer ' + access_token}
     response = requests.get(url, headers=header)
@@ -178,7 +141,7 @@ def get_most_recent_index_tracking_assumption(index: str, access_token: str, ver
         return pd.json_normalize(json.loads(response.text))
     elif response.status_code == 404:
         if verbose:
-            print("Status Code: {}, no tracking data available for {}".format(response.status_code, index))
+            print("Status Code: {}, {}.".format(response.status_code, response.text))
         return pd.DataFrame()
     elif response.status_code == 401:
         if verbose:
